#include <TBranch.h>
#include <TFile.h>
#include <TLeaf.h>
#include <TProfile.h>
#include <TTree.h>
#include <cmath>
#include <iostream>

/*
 *  UTEmuTrimDAC.cpp
 *
 *  Created on: January, 2024
 *      Author: Mingjie Feng (mingjie.feng@cern.ch)
 *
 */

std::vector<std::string> GetChannelNames( TTree* tree ) {
  std::vector<std::string> channelNames;
  TObjArray*               branches = tree->GetListOfBranches();
  for ( int i = 0; i < branches->GetEntries(); ++i ) {
    TBranch* branch = dynamic_cast<TBranch*>( branches->At( i ) );
    if ( branch ) {
      const char* branchName = branch->GetName();
      if ( strstr( branchName, "UT" ) ) { channelNames.push_back( branchName ); }
    }
  }
  return channelNames;
}

int oneChannel( TTree* tree, const char* channel, const char* runNumber, Int_t cut ) {
  std::string channelString( channel );
  size_t      underscorePos     = channelString.find( '_' );
  size_t      lastUnderscorePos = channelString.rfind( '_' );
  std::string layer             = channelString.substr( 0, underscorePos );
  std::string asic              = channelString.substr( 0, lastUnderscorePos );

  TProfile* profile = new TProfile( "profile", "Profile", 256, -0.5, 255.5 );

  // tree->Draw(Form("%s:calibrationStep>>profile", channel), Form("%s != 0", channel), "goff");
  tree->Draw( Form( "%s:calibrationStep>>profile", channel ), "", "goff" );

  Int_t numPoints = profile->GetNbinsX();
  if ( numPoints != cut ) numPoints = cut;

  int validPoints = 0;
  for ( Int_t i = 1; i <= numPoints; ++i ) {
    Double_t x = profile->GetBinCenter( i );

    if ( x < cut ) {
      Double_t y = profile->GetBinContent( i );
      validPoints++;
    }
  }

  Double_t* xValues = new Double_t[validPoints];
  Double_t* yValues = new Double_t[validPoints];

  int currentIndex = 0;
  for ( Int_t i = 1; i <= numPoints; ++i ) {
    Double_t x = profile->GetBinCenter( i );
    if ( x < cut ) {
      Double_t y            = profile->GetBinContent( i );
      xValues[currentIndex] = x;
      yValues[currentIndex] = y;
      currentIndex++;
    }
  }
  delete profile;

  TGraph* graph = new TGraph( validPoints, xValues, yValues );
  graph->GetXaxis()->SetTitle( "Trim DAC Value" );
  graph->GetYaxis()->SetTitle( "Signal Baseline [ADC Value]" );

  // TF1 *fitFunc = new TF1("fitFunc", "pol1", xValues[0], xValues[numPoints-1]);
  TF1* fitFunc = new TF1( "fitFunc", "pol1", 10, 250 );

  auto result = graph->Fit( fitFunc, "QSR" );

  double chi2         = result->Chi2() / result->Ndf();
  double slope        = fitFunc->GetParameter( 1 );
  double intercept    = fitFunc->GetParameter( 0 );
  double intersection = -intercept / slope;

  double slopeError     = fitFunc->GetParError( 1 );
  double interceptError = fitFunc->GetParError( 0 );

  double intersectionError =
      sqrt( pow( interceptError / slope, 2 ) + pow( ( slopeError * intercept ) / ( slope * slope ), 2 ) );

  // std::cout<< validPoints << "Fit parameters: Slope = " << slope << ", Intercept = " << intercept << " , baselineDAC
  // = " << intersection << " Error = " << intersectionError << " , chi2 = " << chi2 << " , Status = " <<
  // result->Status() << std::endl;

  TCanvas* canvas = new TCanvas( channel, channel, 800, 600 );
  graph->Draw( "AP" );
  canvas->Update();

  // TF1 *fitLine = new TF1( channel, Form("%f*x + %f", slope, intercept), xValues[0], xValues[numPoints-1]);
  TF1* fitLine = new TF1( channel, Form( "%f*x + %f", slope, intercept ), 1, 256 );
  fitLine->SetLineColor( kRed );
  fitLine->SetLineWidth( 2 );
  fitLine->Draw( "same" );

  TLine* lineX = new TLine( xValues[0], 0, xValues[numPoints - 1], 0 );
  lineX->SetLineStyle( 2 );
  lineX->Draw( "same" );

  TText* text =
      new TText( 100, 0.75 * graph->GetMaximum(), ( "Best DAC Value = " + std::to_string( intersection ) ).c_str() );
  text->Draw( "same" );

  canvas->Draw();

  const char* filename = Form( "./results/%s/%s/%s/channels.root", runNumber, layer.c_str(), asic.c_str() );
  gSystem->mkdir( gSystem->DirName( filename ), true );

  TFile* plotFile =
      new TFile( Form( "./results/%s/%s/%s/channels.root", runNumber, layer.c_str(), asic.c_str() ), "UPDATE" );
  if ( !plotFile || plotFile->IsZombie() ) {
    std::cerr << "Error opening the plotFile!" << std::endl;
    return 1;
  }
  canvas->Write();
  plotFile->Close();
  delete graph;
  delete text;
  delete lineX;
  delete plotFile;

  int flag = 1;
  if ( chi2 > 20 || chi2 <= 0 || std::isnan( intersection ) || intersection < -200 || intersection > 560 ) flag = 0;

  TFile* rootFile =
      new TFile( Form( "./results/%s/%s/%s/trimDAC.root", runNumber, layer.c_str(), asic.c_str() ), "UPDATE" );
  if ( !rootFile || rootFile->IsZombie() ) {
    std::cerr << "Error opening the rootFile!" << std::endl;
    return 1;
  }
  fitLine->Write();
  rootFile->Close();
  delete fitFunc;
  delete fitLine;
  delete canvas;
  delete rootFile;

  TH2D* histogram2D = new TH2D( channel, channel, 256, -0.5, 255.5, 64, -32.5, 31.5 );
  tree->Draw( Form( "%s:calibrationStep>>%s", channel, channel ), "", "goff" );

  TFile* plot2DFile =
      new TFile( Form( "./results/%s/%s/%s/channel2Ds.root", runNumber, layer.c_str(), asic.c_str() ), "UPDATE" );
  if ( !plot2DFile || plot2DFile->IsZombie() ) {
    std::cerr << "Error opening the plotFile!" << std::endl;
    return 1;
  }

  histogram2D->GetXaxis()->SetTitle( "Trim DAC Value" );
  histogram2D->GetYaxis()->SetTitle( "Signal Baseline [ADC Value]" );
  histogram2D->Draw( "cloz" );
  histogram2D->Write();
  plot2DFile->Close();
  delete histogram2D;
  delete plot2DFile;

  std::ofstream outputFile1( Form( "./results/%s/%s/%s/baselineDAC.csv", runNumber, layer.c_str(), asic.c_str() ),
                             std::ios::app );
  if ( !outputFile1.is_open() || outputFile1.fail() ) {
    std::cerr << "Error opening or writing to the baselineDAC1 output file!" << std::endl;
    return 1;
  }
  if ( intersection > 255 )
    outputFile1 << channel << ", " << 255 << ", " << flag << std::endl;
  else
    outputFile1 << channel << ", " << std::round( intersection ) << ", " << flag << std::endl;
  outputFile1.close();

  std::ofstream outputFile2(
      Form( "./results/%s/%s/%s/baselineDACWithErrors.csv", runNumber, layer.c_str(), asic.c_str() ), std::ios::app );
  if ( !outputFile2.is_open() || outputFile2.fail() ) {
    std::cerr << "Error opening or writing to the baselineDAC1 output file!" << std::endl;
    return 1;
  }
  outputFile2 << channel << ", " << intersection << ", " << intersectionError << std::endl;
  outputFile2.close();

  std::ofstream outputFile_chi2( Form( "./results/%s/%s/%s/fitChi2.csv", runNumber, layer.c_str(), asic.c_str() ),
                                 std::ios::app );
  if ( !outputFile_chi2.is_open() || outputFile_chi2.fail() ) {
    std::cerr << "Error opening or writing to the chi2 output file!" << std::endl;
    return 1;
  }
  outputFile_chi2 << channel << ", " << chi2 << ", " << result->Status() << std::endl;
  outputFile_chi2.close();

  delete[] xValues;
  delete[] yValues;
  return 1;
}

int UT_TrimDAC_plot() {

  // gROOT->ProcessLine(".x ./utilities/lhcbStyle.C");

  gROOT->SetBatch();

  const char* runNumber = "0000284469";
  const char* path      = Form( "/group/ut/ONLINE/tmp/%s/", runNumber );
  const char* layer     = "UTaU";
  const char* fileName  = Form( "%s_merged.root", layer );

  TFile* file = TFile::Open( Form( "%s%s", path, fileName ) );
  if ( !file || file->IsZombie() ) {
    std::cerr << "Error opening the file!" << std::endl;
    return 1;
  }

  TTree* tree = dynamic_cast<TTree*>( file->Get( Form( "%s", layer ) ) );
  if ( !tree ) {
    std::cerr << "Error retrieving the tree!" << std::endl;
    return 1;
  }

  std::vector<std::string> channelNames = GetChannelNames( tree );
  Int_t                    cut          = 256;

  for ( auto channel : channelNames ) { oneChannel( tree, channel.c_str(), runNumber, cut ); }

  delete tree;
  file->Close();

  return 0;
}
